# frozen_string_literal: true

require 'json'
require 'net/http'
require 'active_support/core_ext/module/delegation'

module Sorare
  module Rewards
    # GameWeek stores the reward data of a game week
    class GameWeek
      attr_reader :salt

      delegate :card_allocations, :prize_pools, to: :reward_allocations
      delegate :cards_picked, to: :card_rewards

      def initialize(data: nil, hash: nil, salt: nil)
        @uri = URI("#{Sorare::Rewards.configuration.gateway}#{hash}")
        @data = data
        @salt = salt
      end

      %w[public_seed version supply playing_players leagues].each do |key|
        define_method(key) do
          data[key]
        end
      end

      def data
        @data ||= JSON.parse(::Net::HTTP.get(@uri))
      end

      def config
        @config ||= GameWeekConfig.new(@data['config'])
      end

      def supply_for(league, rarity: nil, player: nil)
        return (supply.dig(league, rarity, player) || 0) if rarity && player
        return (supply.dig(league, rarity) || {}) if rarity

        supply[league] || {}
      end

      def each_league
        (leagues || {}).each_key do |league_name|
          return to_enum(:each_league) unless block_given?

          yield(league(league_name))
        end
      end

      def reward_allocations
        raise 'Salt required' unless salt

        @reward_allocations ||= Sorare::Rewards::Build.call!(game_week: self, salt: salt)
      end

      def card_rewards
        @card_rewards ||= Sorare::Rewards::Pick.call!(reward_allocations.to_h)
      end

      def league(name)
        raise 'Unknown league' unless (leagues || {})[name]

        League.new(
          game_week: self, name: name, config: leagues[name], supply: supply[leagues.dig(name, 'pool_supply')]
        )
      end

      def player(slug)
        Player.new(slug, self)
      end
    end
  end
end
