# frozen_string_literal: true

require 'active_support/core_ext/module/delegation'

module Sorare
  module Rewards
    module Allocations
      # ComputeForQuality computes the reward allocations for a given quality supply between the divisions of a league
      class ComputeForQuality
        include Interactor

        delegate :supply, :config, :randomizer, :allocated, to: :context

        def call
          check_config!

          context.allocated = Array.new(config.length, 0)
          allocate!
          context.quality_allocations = allocations
        end

        def allocations
          config.length.times.map do |division|
            [Sorare::Rewards.configuration.transform_division.call(division + 1), allocated[division]]
          end.to_h
        end

        def allocate!
          allocate_cards
          allocate_pct if has?('pct')
          allocate_extra if has?('pct')
          allocate_loop if has?('cards_per_loop')
        end

        def allocate_divisions(&block)
          config.each_with_index(&block)
        end

        def allocate_cards
          allocate_divisions do |division_config, division|
            count = [remaining_supply, division_config['cards'] || 0].min

            allocated[division] += count
          end
        end

        def allocate_pct
          allocate_divisions do |division_config, division|
            count = [remaining_supply, (supply * (division_config['pct'] || 0)).floor].min

            allocated[division] += count
          end
        end

        def allocate_loop
          while remaining_supply.positive?
            allocate_divisions do |division_config, division|
              allocated[division] += [division_config['cards_per_loop'] || 0, remaining_supply].min
            end
          end
        end

        def allocate_extra
          allocate_divisions do |division_config, division|
            count = pick_extra_supply((supply * (division_config['pct'] || 0)).modulo(1))

            allocated[division] += count
          end
        end

        def remaining_supply
          supply - allocated.sum
        end

        def pick_extra_supply(probability)
          reward_probability = randomizer.rand
          return 0 unless remaining_supply.positive?
          return 0 unless reward_probability < probability

          1
        end

        def has?(key)
          !config.first[key].nil?
        end

        def check_config!
          return if config

          context.fail!(error: 'Invalid config')
        end
      end
    end
  end
end
