# frozen_string_literal: true

require 'active_support/core_ext/module/delegation'
require 'active_support/core_ext/enumerable'

module Sorare
  module Rewards
    module Cards
      # PickForLeague picks the rewards for a given league
      # Receive a game week object
      # And a league allocations
      #   {
      #     'D1' => {
      #       'rare' => { 'tier_0' => 1, 'tier_1' => 2 }
      #     }
      #   }
      class PickForLeague
        include Interactor

        delegate :salt, :league, :allocations, :pickers, to: :context
        delegate :game_week, to: :league

        def call
          context.pickers = {}
          add_pickers!

          context.cards_picked = allocations.keys.index_with do |division|
            PickForDivision.call!(
              **context.to_h, pickers: pickers, allocations: allocations[division]
            ).cards_picked
          end
        end

        def add_pickers!
          league.supply.keys.index_with do |rarity|
            pickers[rarity] = {}
            qualify_and_add!(rarity)
          end
        end

        def qualify_and_add!(rarity)
          sorted_supply = league.supply[rarity]
          qualified_players = Tiers::QualifyPlayers.call!(sorted_supply: sorted_supply).players
          qualified_players.each_with_index.map do |players, tier|
            transformed_tier = Sorare::Rewards.configuration.transform_tier.call(tier)
            initialize_tier_picker(players, rarity, transformed_tier)
          end
        end

        def initialize_tier_picker(players, rarity, tier)
          pickers[rarity][tier] = Picker.new(game_week.public_seed, salt)

          players.each do |slug, _|
            pickers[rarity][tier].add_player(slug, player_supply(rarity, slug))
          end
        end

        def player_supply(rarity, slug)
          game_week.player(slug).overall_supply(league, rarity)
        end
      end
    end
  end
end
