# frozen_string_literal: true

require 'active_support/core_ext/module/delegation'

module Sorare
  module Rewards
    module Tiers
      # QualifyPlayers qualifies a list of sorted supply in tiers based on its rank or a provided tier
      # Returns an array of tiers (array) containing the list of slugs for that tier
      class QualifyPlayers
        include Interactor

        delegate :sorted_supply, to: :context

        def call
          context.players = by_rank ? qualified_players_by_rank : qualified_players_by_tier
        end

        # Use the tier specified within the data
        def qualified_players_by_tier
          Sorare::Rewards.configuration.tiers.times.map do |tier|
            sorted_supply.keys.filter do |slug|
              sorted_supply.dig(slug, 'tier') == tier
            end
          end
        end

        # Use the rank in the array
        def qualified_players_by_rank
          nb_of_tiers.times.map do |tier|
            sorted_supply.keys[tier_index(tier)..tier_index(tier + 1) - 1]
          end
        end

        def tier_depth
          @tier_depth ||= sorted_supply.keys.length / (nb_of_tiers**2 - 1)
        end

        def nb_of_tiers
          @nb_of_tiers ||= begin
            default = Sorare::Rewards.configuration.tiers
            default -= 1 while (default**2 - 1) > sorted_supply.keys.length

            default
          end
        end

        def tier_index(tier)
          return 0 unless tier.positive?
          return sorted_supply.keys.length if tier == nb_of_tiers

          tier_index(tier - 1) + tier_depth * tier_size(tier - 1)
        end

        def tier_size(tier)
          return 1 unless tier.positive?

          tier_size(tier - 1) * 2
        end

        def by_rank
          @by_rank ||= sorted_supply.values.first&.dig('tier').nil?
        end
      end
    end
  end
end
