# frozen_string_literal: true

require 'active_support/core_ext/object/blank'
require 'time'

module Sorare
  module Rewards
    # Player stores the reward data of a player for a game week
    class Player
      attr_reader :slug

      def initialize(slug, reward_data)
        @slug = slug
        @supply = Hash.new { |h, k| h[k] = {} }
        @config = reward_data.config
        @game_data = reward_data.playing_players[slug]
        load_supply(reward_data)
      end

      def rewardable?(league, rarity)
        !@config.cooldown?(rarity, @game_data.dig(league, rarity) || [])
      end

      def playing?
        @game_data.present?
      end

      def reward_pool_supply_contribution(league, rarity)
        return 0 unless playing? && remaining_games_for_supply.positive?

        [
          uncapped_supply_for_game_week(league, rarity),
          pickable_supply(league, rarity)
        ].min
      end

      def pickable_supply(league, rarity)
        [
          overall_supply(league, rarity), # Remaining supply
          available_during_cooldown(league, rarity), # Cooldown limit
          @config.distribution_limit(rarity) # Hard limit
        ].min
      end

      def rank(league, rarity)
        @supply.dig(league.name, rarity, 'rank')
      end

      def overall_supply(league, rarity)
        @supply.dig(league.name, rarity, 'supply') || 0
      end

      def uncapped_supply_for_game_week(league, rarity)
        overall_supply(league, rarity) / remaining_games_for_supply.to_f
      end

      def available_during_cooldown(league, rarity)
        [@config.cooldown_limit(rarity) - rewarded_during_cooldown(league, rarity), 0].max
      end

      def rewarded_during_cooldown(league, rarity)
        (@game_data&.dig(league.name, rarity) || []).count do |at|
          (Time.parse(at) + @config.cooldown_since(rarity)) > Time.now
        end
      end

      %w[remaining_games_for_supply remaining_games].each do |key|
        define_method(key) do
          @game_data&.dig(key)
        end
      end

      private

      def load_supply(reward_data)
        reward_data.each_league do |league|
          league.each_rarity_supply do |rarity, supply|
            next unless supply[slug]

            @supply[league.name][rarity] = supply[slug]
          end
        end
      end
    end
  end
end
